package ch.hei.kart;

import java.util.EnumSet;

/**
 * Kart hardware settings.
 * 
 * @author Michael Clausen (clm@hevs.ch)
 */
public enum KartHardwareSettings {
	/**
	 * If this option is set, the Kart inverses the drive motor direction.
	 */
	InverseDriveMotorDirection,

	/**
	 * If this option is set, the Kart inverses the steering stepper motor
	 * direction.
	 */
	InverseSteeringDirection,

	/**
	 * If this option is set, the Kart inverses the steering stepper motor
	 * counter direction as the end sensor is on the opposite position.
	 */
	InverseSteeringEndContactPosition,

	/**
	 * If this option is set, the kart does not use the local steering end
	 * switch contact, it uses a simulated one.
	 */
	EmulateSteeringEndContact,

	/**
	 * Once this option is set, some of the slave FPGA's are reseted.
	 */
	RestartSlaves;

	// Converts the enumeration set to an integer value.
	static int toInt(EnumSet<KartHardwareSettings> settings) {
		int value = 7;
		for (KartHardwareSettings setting : settings) {
			switch (setting) {
			case InverseDriveMotorDirection:
				value &= ~1;
				break;

			case InverseSteeringDirection:
				value &= ~2;
				break;

			case InverseSteeringEndContactPosition:
				value += ~4;
				break;

			case EmulateSteeringEndContact:
				value += 8;
				break;

			case RestartSlaves:
				value += 16;
				break;
			}
		}

		return value;
	}

	// Converts an integer value to a corresponding enum set of this type.
	static EnumSet<KartHardwareSettings> fromInt(int value) {
		EnumSet<KartHardwareSettings> set = EnumSet
				.noneOf(KartHardwareSettings.class);
		if ((value & 1) != 0)
			set.add(KartHardwareSettings.InverseDriveMotorDirection);
		if ((value & 2) != 0)
			set.add(KartHardwareSettings.InverseSteeringDirection);
		if ((value & 4) != 0)
			set.add(KartHardwareSettings.InverseSteeringEndContactPosition);
		if ((value & 8) != 0)
			set.add(KartHardwareSettings.EmulateSteeringEndContact);
		if ((value & 16) != 0)
			set.add(KartHardwareSettings.RestartSlaves);
		return set;
	}
}
