package ch.hei.kart.comm.discovery;

import android.content.Context;

/**
 * Kart discovery interface. 
 * 
 * All transport protocols that support Kart discovery have to implement this interface.
 * 
 * @author Michael Clausen (clm@hevs.ch)
 */
public interface Discovery {
	/**
	 * Starts the actual discovery process using the given listener and Android context.
	 * 
	 * @param listener	The object to get informed about new found entities implementing the 
	 * 					DiscoveryListener interface.
	 * @param context	The actual Android context (some transports might need it).
	 * @return			True if the discovery process could be started, false otherwise.
	 */
	public boolean startDiscovery(DiscoveryListener listener, Context context);
	
	/**
	 * Stops an active Kart entity discovery process.
	 */
	public void stopDiscovery();
}
