package ch.hei.kart.comm.discovery;

import java.io.Serializable;
import java.util.List;

import ch.hei.kart.comm.protocol.Protocol;
import ch.hei.kart.comm.transport.Transport;

/**
 * Minimal interface a discovered Kart entity has to implement.
 * 
 * @author Michael Clausen (clm@hevs.ch)
 */
public interface DiscoveryEntity extends Serializable {
	/**
	 * Returns the user friendly name of the Kart entity.
	 * 
	 * @return Userfriendly name.
	 */
	public String getFriendlyName();
	
	/**
	 * Returns a textual description of the Kart entity. This is mostly in the form
	 * 		  "Protocol - Address".
	 * 
	 * @return Entitiy description.
	 */
	public String getDescription();
	
	/**
	 * Returns the class of the transport implementation that has to be instantiated in
	 * 	      order to to create a transport connection to the entity.
	 * 
	 * @return Transport implementation class.
	 */
	public Class<? extends Transport> getTransportImplementation();
	
	/**
	 * Returns a list of supported protocols of the entity using the transport protocol 
	 * 		  returned by the method getTransportImplementation().
	 * 
	 * @return List of supported protocol implementation classes.
	 */
	public List<Class<? extends Protocol>> getSupportedProtocols();
	
	/**
	 * Checks for equality of another entity with this.
	 * 
	 * @param other	Reference to the other entity to compare this with.
	 * @return True if the entity objects are equal, false otherwise.
	 */
	public boolean equals(DiscoveryEntity other);
}
