package ch.hei.kart.comm.protocol;

import ch.hei.kart.KartStatusRegister;

/**
 * Interface used by protocol objects implementing the Protocol interface in order to 
 * 		  inform any client class about register changes or communication state/errors.
 *  
 * @author Michael Clausen (clm@hevs.ch)
 */
public interface ProtocolListener {
	/**
	 * Called by the protocol in order to notify the observing object about the update of 
	 * 		  a register.
	 * 
	 * @param protocol	The protocol object actually notifying.
	 * @param register	The Kart status register that has changed.
	 * @param value		The new value to set for the register.
	 */
	void registerValueIndication(Protocol protocol, KartStatusRegister register, int value);
	
	/**
	 * Called to signal state changes in the connection to the Kart.
	 * 
	 * @param protocol	The protocol object actually notifying. 
	 * @param state		The new state of the protocol.
	 */
	void stateChangeIndication(Protocol protocol, ProtocolState state);
	
	/**
	 * Called whenever there was an error in the protocol or the link to the Kart.
	 * 
	 * @param protocol	The protocol object actually notifying. 
	 * @param error		A description of the error as a string.
	 */
	void errorIndication(Protocol protocol, String error);
}
