package ch.hei.kart.comm.transport.bluetooth;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.widget.Toast;
import ch.hei.kart.R;
import ch.hei.kart.comm.discovery.Discovery;
import ch.hei.kart.comm.discovery.DiscoveryListener;

public class RfCommDiscovery implements Discovery {

	private DiscoveryListener mListener;
	private Context mContext;

	@Override
	public boolean startDiscovery(DiscoveryListener listener, Context context) {
		if (listener == null || context == null)
			return false;
		mListener = listener;
		mContext = context;

		btAdapter = BluetoothAdapter.getDefaultAdapter();
		if (btAdapter == null)
			return false;

		IntentFilter filter = new IntentFilter(BluetoothDevice.ACTION_FOUND);
		filter.addAction(BluetoothAdapter.ACTION_STATE_CHANGED);
		context.registerReceiver(btReceiver, filter);

		// Bluetooth must be enable before starting the discovery. Enable it if
		// necessary (the first time).
		if (!btAdapter.isEnabled()) {
			btAdapter.enable();
			Toast.makeText(mContext, R.string.start_bluetooth,
					Toast.LENGTH_SHORT).show();
			// Start the discovery when Bluetooth is enabled (after
			// ACTION_STATE_CHANGED)
			return true;
		}
		return btAdapter.startDiscovery(); // Successful or not
	}

	@Override
	public void stopDiscovery() {
		mContext.unregisterReceiver(btReceiver);
		btAdapter.cancelDiscovery();
	}

	private BluetoothAdapter btAdapter;
	private BroadcastReceiver btReceiver = new BroadcastReceiver() {
		@Override
		public void onReceive(Context context, Intent intent) {
			if (intent.getAction().equals(BluetoothDevice.ACTION_FOUND)) {
				BluetoothDevice device;
				device = intent
						.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
				if (device.getName() != null && 
						device.getName().toLowerCase().contains("kart") && 
						device.getName().toLowerCase().contains("#")) {
					if (mListener != null)
						mListener.entityFoundIndication(RfCommDiscovery.this,
								new RfCommDiscoveryEntity(device));
				}
			} else if (intent.getAction().equals(
					BluetoothAdapter.ACTION_STATE_CHANGED)) {
				// Bluetooth is now activated. Force discovering.
				if(btAdapter.getState() == BluetoothAdapter.STATE_ON)
					btAdapter.startDiscovery();
			}
		}
	};
}
