package ch.hei.kart.comm.transport.tcp;

import java.io.IOException;
import java.io.InputStream;
import java.io.InvalidClassException;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.nio.ByteBuffer;

import ch.hei.kart.comm.discovery.DiscoveryEntity;
import ch.hei.kart.comm.transport.Transport;
import ch.hei.kart.comm.transport.TransportListener;
import ch.hei.kart.comm.transport.TransportState;

public class TcpTransport implements Transport {
	@Override
	public void connect(String connectionString) throws Exception {
		tcpSocket = new Socket();
		
		tcpSocket.connect(new InetSocketAddress(connectionString, 0x0444));
		
		tcpInput = tcpSocket.getInputStream();
		tcpOutput = tcpSocket.getOutputStream();

		tcpReader.start();
	}
	
	public void connect(DiscoveryEntity entity) throws Exception {
		if (entity.getTransportImplementation() != this.getClass() || 
			entity.getClass() != TcpDiscoveryEntity.class) 
			throw new InvalidClassException("Kart entity not suited for transport!");
		
		connect(((TcpDiscoveryEntity)entity).getHost().getHostAddress());
	}

	@Override
	public boolean isConnected() {
		return tcpSocket != null && tcpSocket.isConnected();
	}

	@Override
	public void disconnect() {
		if (tcpSocket == null) return;
		
		tcpInput = null;
		tcpOutput = null;
		if (tcpSocket != null) {
			try {
				tcpSocket.close();
			} catch (IOException exception) {
				if (listener != null) listener.errorIndication(this, exception.getMessage());
				exception.printStackTrace();
			}
			tcpSocket = null;
		}
		
		setState(TransportState.ConnectionLost);
		listener = null;
	}
	
	@Override
	public void setListener(TransportListener listener) {
		this.listener = listener;
	}

	@Override
	public boolean write(ByteBuffer buffer) {
		if (tcpSocket == null || tcpOutput == null || ! tcpSocket.isConnected()) return false;
		
		try {
			buffer.flip();
			byte[] data = new byte[buffer.remaining()];
			buffer.get(data);
			
			tcpOutput.write(data);
			return true;
		} catch (Exception exception) {
			if (listener != null) listener.errorIndication(this, exception.getMessage());
			exception.printStackTrace();
			disconnect();
			return false;
		}
	}

	private Socket tcpSocket;
	private InputStream tcpInput;
	private OutputStream tcpOutput;
	private Thread tcpReader = new Thread() {
		@Override
		public void run() {
			byte[] buffer = new byte[128];
			while(tcpInput != null) {
				try {
					int size = tcpInput.read(buffer);
					if (size <= 0) {
						disconnect();
						return;
					}
					ByteBuffer byteBuffer = ByteBuffer.allocate(size);
					byteBuffer.put(buffer, 0, size);
					byteBuffer.flip();
					if (listener != null) listener.dataIndication(TcpTransport.this, byteBuffer);
				} catch (IOException exception) {
					if (tcpInput != null) {
						if (listener != null) listener.errorIndication(TcpTransport.this, 
																   	   exception.getMessage());
						exception.printStackTrace();
					}
					disconnect();
				}
			}
		}
	};
	
	private TransportListener listener = null;
	
	private TransportState state = TransportState.Idle;
	
	private void setState(TransportState state) {
		if (state != this.state) {
			this.state = state;
			if (listener != null) 
				listener.stateChangeIndication(this, state);
		}
	}
}
