package ch.hei.kart.ui;

import java.util.List;

import android.app.Activity;
import android.content.Intent;
import android.content.pm.ResolveInfo;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import ch.hei.kart.R;
import ch.hei.kart.comm.CommunicationSetup;
import ch.hei.kart.comm.discovery.Discovery;
import ch.hei.kart.comm.discovery.DiscoveryEntity;
import ch.hei.kart.comm.discovery.DiscoveryListener;

/**
 * Activity for scanning and displaying available Kart devices.
 * 
 * @author Christopher Metrailler (mei@hevs.ch)
 * @author Michael Clausen (clm@hevs.ch)
 * 
 * @version 2.0 - 2014
 */
public class EntityScanActivity extends Activity implements DiscoveryListener, OnItemClickListener {

	private ListView mListView;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		// Get all available RCA Activities
		Intent intent = new Intent(Intent.ACTION_RUN);
		intent.addCategory(AbstractKartControlActivity.INTENT_CATEGORY);
		rcActivities = getPackageManager().queryIntentActivities(intent, 0);

		// ListView Activity setup
		setContentView(R.layout.discrover_layout);
		mListView = (ListView) findViewById(R.id.list);
		mListView.setEmptyView(findViewById(R.id.empty));
		mListView.setOnItemClickListener(this);
		
		// Initializes list view adapter.
		adapter = new EntityListAdapter(this);
		mListView.setAdapter(adapter);
		registerForContextMenu(mListView);
	}

	@Override
	protected void onResume() {
		super.onResume();
		scanDevice(true);
	}

	@Override
	protected void onPause() {
		if (scanningInprocess)
			scanDevice(false);
		adapter.clear();
		super.onPause();
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		if (scanningInprocess)
			scanDevice(false);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.main, menu);
		if (!scanningInprocess) {
			menu.findItem(R.id.menu_stop).setVisible(false);
			menu.findItem(R.id.menu_scan).setVisible(true);
			menu.findItem(R.id.menu_refresh).setActionView(null);
		} else {
			menu.findItem(R.id.menu_stop).setVisible(true);
			menu.findItem(R.id.menu_scan).setVisible(false);
			menu.findItem(R.id.menu_refresh).setActionView(
					R.layout.actionbar_indeterminate_progress);
		}
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menu_scan:
			adapter.clear();
			scanDevice(true);
			break;
		case R.id.menu_stop:
			scanDevice(false);
			break;
		}
		return true;
	}

	private void scanDevice(final boolean enable) {
		if (enable) {
			scanningInprocess = true;
			CommunicationSetup.startDiscovery(this, this);
		} else {
			CommunicationSetup.stopDiscovery();
			scanningInprocess = false;
		}
		invalidateOptionsMenu();
	}

	@Override
	public void onItemClick(AdapterView<?> parent, View view, int position,
			long id) {
		entity = adapter.getEntity(position);
		if (entity == null)
			return;

		if (rcActivities.isEmpty())
			Toast.makeText(this, R.string.error_no_activity, Toast.LENGTH_LONG).show();
		else if (rcActivities.size() == 1)
			startControlActivity(rcActivities.get(0).activityInfo.name);
		else
			openContextMenu(mListView);
	}

	@Override
	public void entityFoundIndication(Discovery discovery, DiscoveryEntity entity) {
		adapter.addEntity(entity);
	}

	@Override
	public void errorIndication(Discovery discovery, String error) {
		Toast.makeText(this, error, Toast.LENGTH_LONG).show();
	}

	@Override
	public void onCreateContextMenu(ContextMenu menu, View view, ContextMenuInfo menuInfo) {
		super.onCreateContextMenu(menu, view, menuInfo);
		// Display available rc Activities
		for(int i = 0; i < rcActivities.size(); i++) {
			final ResolveInfo info = rcActivities.get(i);
			try {
				final Class<?> c = Class.forName(info.activityInfo.name);
				menu.add(0, i, i, c.getSimpleName());
			} catch (ClassNotFoundException e) {
				// Do nothing
			}			
		}
	}

	@Override
	public boolean onContextItemSelected(MenuItem item) {
		startControlActivity(rcActivities.get(item.getItemId()).activityInfo.name);
		return true;
	}

	private void startControlActivity(String className) {
		final Intent intent = new Intent();
		intent.setClassName(this, className);
		intent.putExtra(AbstractKartControlActivity.EXTRAS_ENTITY, entity);
		startActivity(intent);
	}

	private EntityListAdapter adapter;
	private boolean scanningInprocess = false;
	DiscoveryEntity entity;
	private List<ResolveInfo> rcActivities;

	static class ViewHolder {
		TextView deviceName;
		TextView deviceAddress;
		ImageView icon;
	}
}